@@ -108,7 +108,7 @@ fn main() {
         TreeSitterParser {
             name: "tree-sitter-cpp",
             src_dir: "vendored_parsers/tree-sitter-cpp-src",
-            extra_files: vec!["scanner.cc"],
+            extra_files: vec!["scanner.c"],
         },
         TreeSitterParser {
             name: "tree-sitter-c-sharp",
@@ -230,6 +230,11 @@ fn main() {
             src_dir: "vendored_parsers/tree-sitter-kotlin-src",
             extra_files: vec!["scanner.c"],
         },
+        TreeSitterParser {
+            name: "tree-sitter-latex",
+            src_dir: "vendored_parsers/tree-sitter-latex-src",
+            extra_files: vec!["scanner.c"],
+        },
         TreeSitterParser {
             name: "tree-sitter-lua",
             src_dir: "vendored_parsers/tree-sitter-lua-src",
