@@ -5,13 +5,13 @@ use std::{cmp::Reverse, env};
 
 use crate::{
     diff::changes::ChangeMap,
-    diff::graph::{get_set_neighbours, populate_change_map, Edge, Vertex},
+    diff::graph::{populate_change_map, set_neighbours, Edge, Vertex},
+    hash::DftHashMap,
     parse::syntax::Syntax,
 };
 use bumpalo::Bump;
 use itertools::Itertools;
 use radix_heap::RadixHeapMap;
-use rustc_hash::FxHashMap;
 
 #[derive(Debug)]
 pub struct ExceededGraphLimit {}
@@ -30,7 +30,7 @@ fn shortest_vertex_path<'a, 'b>(
 
     heap.push(Reverse(0), start);
 
-    let mut seen = FxHashMap::default();
+    let mut seen = DftHashMap::default();
     seen.reserve(size_hint);
 
     let end: &'b Vertex<'a, 'b> = loop {
@@ -40,7 +40,8 @@ fn shortest_vertex_path<'a, 'b>(
                     break current;
                 }
 
-                for neighbour in &get_set_neighbours(current, vertex_arena, &mut seen) {
+                set_neighbours(current, vertex_arena, &mut seen);
+                for neighbour in current.neighbours.borrow().as_ref().unwrap() {
                     let (edge, next) = neighbour;
                     let distance_to_next = distance + edge.cost();
 
@@ -257,14 +258,6 @@ mod tests {
         }]
     }
 
-    fn col_helper(line: u32, col: u32) -> Vec<SingleLineSpan> {
-        vec![SingleLineSpan {
-            line: line.into(),
-            start_col: col,
-            end_col: col + 1,
-        }]
-    }
-
     #[test]
     fn identical_atoms() {
         let arena = Arena::new();
@@ -282,6 +275,7 @@ mod tests {
         assert_eq!(
             actions,
             vec![UnchangedNode {
+                probably_punctuation: false,
                 depth_difference: 0
             }]
         );
@@ -326,10 +320,7 @@ mod tests {
                 EnterUnchangedDelimiter {
                     depth_difference: 0
                 },
-                NovelAtomLHS {
-                    contiguous: false,
-                    probably_punctuation: false,
-                },
+                NovelAtomLHS {},
             ]
         );
     }
@@ -371,14 +362,8 @@ mod tests {
                 EnterUnchangedDelimiter {
                     depth_difference: 0
                 },
-                NovelAtomRHS {
-                    contiguous: false,
-                    probably_punctuation: false
-                },
-                NovelAtomRHS {
-                    contiguous: false,
-                    probably_punctuation: false
-                },
+                NovelAtomRHS {},
+                NovelAtomRHS {},
             ]
         );
     }
@@ -420,142 +405,20 @@ mod tests {
         assert_eq!(
             actions,
             vec![
-                EnterNovelDelimiterRHS { contiguous: false },
-                EnterNovelDelimiterLHS { contiguous: false },
+                EnterNovelDelimiterRHS {},
+                EnterNovelDelimiterLHS {},
                 UnchangedNode {
+                    probably_punctuation: false,
                     depth_difference: 0
                 },
                 UnchangedNode {
+                    probably_punctuation: false,
                     depth_difference: 0
                 },
             ],
         );
     }
 
-    #[test]
-    fn prefer_atoms_same_line() {
-        let arena = Arena::new();
-
-        let lhs = vec![
-            Syntax::new_atom(&arena, col_helper(1, 0), "foo", AtomKind::Normal),
-            Syntax::new_atom(&arena, col_helper(2, 0), "bar", AtomKind::Normal),
-            Syntax::new_atom(&arena, col_helper(2, 1), "foo", AtomKind::Normal),
-        ];
-
-        let rhs = vec![Syntax::new_atom(
-            &arena,
-            col_helper(1, 0),
-            "foo",
-            AtomKind::Normal,
-        )];
-        init_all_info(&lhs, &rhs);
-
-        let start = Vertex::new(lhs.get(0).copied(), rhs.get(0).copied());
-        let vertex_arena = Bump::new();
-        let route = shortest_path(start, &vertex_arena, 0, DEFAULT_GRAPH_LIMIT).unwrap();
-
-        let actions = route.iter().map(|(action, _)| *action).collect_vec();
-        assert_eq!(
-            actions,
-            vec![
-                UnchangedNode {
-                    depth_difference: 0
-                },
-                NovelAtomLHS {
-                    contiguous: false,
-                    probably_punctuation: false
-                },
-                NovelAtomLHS {
-                    contiguous: true,
-                    probably_punctuation: false
-                },
-            ]
-        );
-    }
-
-    #[test]
-    fn prefer_children_same_line() {
-        let arena = Arena::new();
-
-        let lhs = vec![Syntax::new_list(
-            &arena,
-            "[",
-            col_helper(1, 0),
-            vec![Syntax::new_atom(
-                &arena,
-                col_helper(1, 2),
-                "1",
-                AtomKind::Normal,
-            )],
-            "]",
-            pos_helper(2),
-        )];
-
-        let rhs = vec![];
-        init_all_info(&lhs, &rhs);
-
-        let start = Vertex::new(lhs.get(0).copied(), rhs.get(0).copied());
-        let vertex_arena = Bump::new();
-        let route = shortest_path(start, &vertex_arena, 0, DEFAULT_GRAPH_LIMIT).unwrap();
-
-        let actions = route.iter().map(|(action, _)| *action).collect_vec();
-        assert_eq!(
-            actions,
-            vec![
-                EnterNovelDelimiterLHS { contiguous: false },
-                NovelAtomLHS {
-                    contiguous: true,
-                    probably_punctuation: false
-                },
-            ]
-        );
-    }
-
-    #[test]
-    fn atom_after_novel_list_contiguous() {
-        let arena = Arena::new();
-
-        let lhs = vec![
-            Syntax::new_list(
-                &arena,
-                "[",
-                col_helper(1, 0),
-                vec![Syntax::new_atom(
-                    &arena,
-                    col_helper(1, 2),
-                    "1",
-                    AtomKind::Normal,
-                )],
-                "]",
-                col_helper(2, 1),
-            ),
-            Syntax::new_atom(&arena, col_helper(2, 2), ";", AtomKind::Normal),
-        ];
-
-        let rhs = vec![];
-        init_all_info(&lhs, &rhs);
-
-        let start = Vertex::new(lhs.get(0).copied(), rhs.get(0).copied());
-        let vertex_arena = Bump::new();
-        let route = shortest_path(start, &vertex_arena, 0, DEFAULT_GRAPH_LIMIT).unwrap();
-
-        let actions = route.iter().map(|(action, _)| *action).collect_vec();
-        assert_eq!(
-            actions,
-            vec![
-                EnterNovelDelimiterLHS { contiguous: false },
-                NovelAtomLHS {
-                    contiguous: true,
-                    probably_punctuation: false
-                },
-                NovelAtomLHS {
-                    contiguous: true,
-                    probably_punctuation: true
-                },
-            ]
-        );
-    }
-
     #[test]
     fn replace_similar_comment() {
         let arena = Arena::new();
@@ -658,10 +521,7 @@ mod tests {
                 ReplacedComment {
                     levenshtein_pct: 95
                 },
-                NovelAtomLHS {
-                    contiguous: false,
-                    probably_punctuation: false
-                }
+                NovelAtomLHS {}
             ]
         );
     }
