@@ -2,14 +2,15 @@
 
 #![allow(clippy::mutable_key_type)] // Hash for Syntax doesn't use mutable fields.
 
-use std::{cell::Cell, collections::HashMap, env, fmt, hash::Hash, num::NonZeroU32};
+use std::{cell::Cell, env, fmt, hash::Hash, num::NonZeroU32};
 use typed_arena::Arena;
 
 use crate::{
     diff::changes::ChangeKind,
     diff::changes::{ChangeKind::*, ChangeMap},
     diff::myers_diff,
-    lines::{is_all_whitespace, LineNumber, NewlinePositions},
+    hash::DftHashMap,
+    lines::{is_all_whitespace, NewlinePositions},
     positions::SingleLineSpan,
 };
 use Syntax::*;
@@ -21,9 +22,16 @@ impl<'a> fmt::Debug for ChangeKind<'a> {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         let desc = match self {
             Unchanged(node) => format!("Unchanged(ID: {})", node.id()),
-            ReplacedComment(lhs_node, rhs_node) => {
+            ReplacedComment(lhs_node, rhs_node) | ReplacedString(lhs_node, rhs_node) => {
+                let change_kind = if let ReplacedComment(_, _) = self {
+                    "ReplacedComment"
+                } else {
+                    "ReplacedString"
+                };
+
                 format!(
-                    "ReplacedComment(lhs ID: {}, rhs ID: {})",
+                    "{}(lhs ID: {}, rhs ID: {})",
+                    change_kind,
                     lhs_node.id(),
                     rhs_node.id()
                 )
@@ -47,9 +55,6 @@ pub struct SyntaxInfo<'a> {
     prev: Cell<Option<&'a Syntax<'a>>>,
     /// The parent syntax node, if present.
     parent: Cell<Option<&'a Syntax<'a>>>,
-    /// Does the previous syntax node occur on the same line as the
-    /// first line of this node?
-    prev_is_contiguous: Cell<bool>,
     /// The number of nodes that are ancestors of this one.
     num_ancestors: Cell<u32>,
     pub num_after: Cell<usize>,
@@ -73,7 +78,6 @@ impl<'a> SyntaxInfo<'a> {
             next_sibling: Cell::new(None),
             prev: Cell::new(None),
             parent: Cell::new(None),
-            prev_is_contiguous: Cell::new(false),
             num_ancestors: Cell::new(0),
             num_after: Cell::new(0),
             unique_id: Cell::new(NonZeroU32::new(u32::MAX).unwrap()),
@@ -277,10 +281,6 @@ impl<'a> Syntax<'a> {
         self.info().next_sibling.get()
     }
 
-    pub fn prev_is_contiguous(&self) -> bool {
-        self.info().prev_is_contiguous.get()
-    }
-
     /// A unique ID of this syntax node. Every node is guaranteed to
     /// have a different value.
     pub fn id(&self) -> SyntaxId {
@@ -328,22 +328,6 @@ impl<'a> Syntax<'a> {
             }
         }
     }
-
-    pub fn first_line(&self) -> Option<LineNumber> {
-        let position = match self {
-            List { open_position, .. } => open_position,
-            Atom { position, .. } => position,
-        };
-        position.first().map(|lp| lp.line)
-    }
-
-    pub fn last_line(&self) -> Option<LineNumber> {
-        let position = match self {
-            List { close_position, .. } => close_position,
-            Atom { position, .. } => position,
-        };
-        position.last().map(|lp| lp.line)
-    }
 }
 
 pub fn comment_positions<'a>(nodes: &[&'a Syntax<'a>]) -> Vec<SingleLineSpan> {
@@ -382,7 +366,7 @@ fn init_info<'a>(lhs_roots: &[&'a Syntax<'a>], rhs_roots: &[&'a Syntax<'a>]) {
     init_info_on_side(lhs_roots, &mut id);
     init_info_on_side(rhs_roots, &mut id);
 
-    let mut existing = HashMap::new();
+    let mut existing = DftHashMap::default();
     set_content_id(lhs_roots, &mut existing);
     set_content_id(rhs_roots, &mut existing);
 
@@ -392,7 +376,7 @@ fn init_info<'a>(lhs_roots: &[&'a Syntax<'a>], rhs_roots: &[&'a Syntax<'a>]) {
 
 type ContentKey = (Option<String>, Option<String>, Vec<u32>, bool, bool);
 
-fn set_content_id(nodes: &[&Syntax], existing: &mut HashMap<ContentKey, u32>) {
+fn set_content_id(nodes: &[&Syntax], existing: &mut DftHashMap<ContentKey, u32>) {
     for node in nodes {
         let key: ContentKey = match node {
             List {
@@ -457,7 +441,6 @@ pub fn init_next_prev<'a>(roots: &[&'a Syntax<'a>]) {
     set_prev_sibling(roots);
     set_next_sibling(roots);
     set_prev(roots, None);
-    set_prev_is_contiguous(roots);
 }
 
 /// Set all the `SyntaxInfo` values for all the `roots` on a single
@@ -481,7 +464,7 @@ fn set_unique_id(nodes: &[&Syntax], next_id: &mut SyntaxId) {
 }
 
 /// Assumes that `set_content_id` has already run.
-fn find_nodes_with_unique_content(nodes: &[&Syntax], counts: &mut HashMap<u32, usize>) {
+fn find_nodes_with_unique_content(nodes: &[&Syntax], counts: &mut DftHashMap<u32, usize>) {
     for node in nodes {
         *counts.entry(node.content_id()).or_insert(0) += 1;
         if let List { children, .. } = node {
@@ -490,7 +473,7 @@ fn find_nodes_with_unique_content(nodes: &[&Syntax], counts: &mut HashMap<u32, u
     }
 }
 
-fn set_content_is_unique_from_counts(nodes: &[&Syntax], counts: &HashMap<u32, usize>) {
+fn set_content_is_unique_from_counts(nodes: &[&Syntax], counts: &DftHashMap<u32, usize>) {
     for node in nodes {
         let count = counts
             .get(&node.content_id())
@@ -504,7 +487,7 @@ fn set_content_is_unique_from_counts(nodes: &[&Syntax], counts: &HashMap<u32, us
 }
 
 fn set_content_is_unique(nodes: &[&Syntax]) {
-    let mut counts = HashMap::new();
+    let mut counts = DftHashMap::default();
     find_nodes_with_unique_content(nodes, &mut counts);
     set_content_is_unique_from_counts(nodes, &counts);
 }
@@ -565,35 +548,6 @@ fn set_num_ancestors(nodes: &[&Syntax], num_ancestors: u32) {
     }
 }
 
-fn set_prev_is_contiguous(roots: &[&Syntax]) {
-    for node in roots {
-        let is_contiguous = if let Some(prev) = node.info().prev.get() {
-            match prev {
-                List {
-                    open_position,
-                    close_position,
-                    ..
-                } => {
-                    let prev_is_parent = prev.num_ancestors() < node.num_ancestors();
-                    if prev_is_parent {
-                        open_position.last().map(|p| p.line) == node.first_line()
-                    } else {
-                        // predecessor node at the same level.
-                        close_position.last().map(|p| p.line) == node.first_line()
-                    }
-                }
-                Atom { .. } => prev.last_line() == node.first_line(),
-            }
-        } else {
-            false
-        };
-        node.info().prev_is_contiguous.set(is_contiguous);
-        if let List { children, .. } = node {
-            set_prev_is_contiguous(children);
-        }
-    }
-}
-
 impl<'a> PartialEq for Syntax<'a> {
     fn eq(&self, other: &Self) -> bool {
         debug_assert!(self.content_id() > 0);
@@ -662,21 +616,33 @@ pub struct MatchedPos {
 /// Split `s` into a vec of things that look like words and individual
 /// non-word characters.
 ///
-/// "foo bar" -> vec!["foo", " ", "bar"]
+/// "foo..bar23" -> vec!["foo", ".", ".", "bar23"]
+///
+/// See also `split_words_and_numbers`. Both these functions are hot,
+/// so they are separate implementations rather than passing a bool to
+/// customise number handling.
 pub fn split_words(s: &str) -> Vec<&str> {
     let mut res = vec![];
-    let mut word_start = None;
+    let mut word_start: Option<usize> = None;
     for (idx, c) in s.char_indices() {
-        if c.is_alphanumeric() || c == '-' || c == '_' {
-            if word_start.is_none() {
-                word_start = Some(idx);
+        match word_start {
+            Some(start) => {
+                if c.is_alphanumeric() || c == '-' || c == '_' {
+                    // Just carry on in this word.
+                } else {
+                    // Push the previous word, then this non-word character.
+                    res.push(&s[start..idx]);
+                    res.push(&s[idx..idx + c.len_utf8()]);
+                    word_start = None;
+                }
             }
-        } else {
-            if let Some(start) = word_start {
-                res.push(&s[start..idx]);
-                word_start = None;
+            None => {
+                if c.is_alphanumeric() || c == '-' || c == '_' {
+                    word_start = Some(idx);
+                } else {
+                    res.push(&s[idx..idx + c.len_utf8()]);
+                }
             }
-            res.push(&s[idx..idx + c.len_utf8()]);
         }
     }
 
@@ -686,16 +652,81 @@ pub fn split_words(s: &str) -> Vec<&str> {
     res
 }
 
-fn split_comment_words(
+/// Split `s` into a vec of things that look like words and individual
+/// non-word characters.
+///
+/// "foo..bar23" -> vec!["foo", ".", ".", "bar23"]
+pub fn split_words_and_numbers(s: &str) -> Vec<&str> {
+    let mut res = vec![];
+    let mut word_start: Option<(usize, char)> = None;
+    for (idx, c) in s.char_indices() {
+        match word_start {
+            Some((start, start_c)) => {
+                if c.is_alphanumeric() || c == '_' {
+                    // Word character, add to the current word if it's
+                    // not a number.
+                    if c.is_ascii_digit() == start_c.is_ascii_digit() {
+                        // Just carry on in this word.
+                    } else {
+                        // Finish previous word, start a new one.
+                        res.push(&s[start..idx]);
+                        word_start = Some((idx, c));
+                    }
+                } else {
+                    // Push the previous word, then this non-word character.
+                    res.push(&s[start..idx]);
+                    res.push(&s[idx..idx + c.len_utf8()]);
+                    word_start = None;
+                }
+            }
+            None => {
+                if c.is_alphanumeric() || c == '-' || c == '_' {
+                    word_start = Some((idx, c));
+                } else {
+                    res.push(&s[idx..idx + c.len_utf8()]);
+                }
+            }
+        }
+    }
+
+    if let Some((start, _)) = word_start {
+        res.push(&s[start..]);
+    }
+    res
+}
+
+/// Given the text `content` from a comment or strings, split it into
+/// MatchedPos values for the novel and unchanged words.
+///
+/// If there is negligible text in common with `opposite_content`,
+/// treat the whole `content` as a single novel region.
+fn split_atom_words(
     content: &str,
-    pos: SingleLineSpan,
+    pos: &[SingleLineSpan],
     opposite_content: &str,
-    opposite_pos: SingleLineSpan,
+    opposite_pos: &[SingleLineSpan],
+    kind: AtomKind,
 ) -> Vec<MatchedPos> {
+    debug_assert!(kind == AtomKind::Comment || kind == AtomKind::String);
+
     // TODO: merge adjacent single-line comments unless there are
     // blank lines between them.
-    let content_parts = split_words(content);
-    let other_parts = split_words(opposite_content);
+    let content_parts = split_words_and_numbers(content);
+    let other_parts = split_words_and_numbers(opposite_content);
+
+    let word_diffs = myers_diff::slice_by_hash(&content_parts, &other_parts);
+
+    if !has_common_words(&word_diffs) {
+        return pos
+            .iter()
+            .map(|line| MatchedPos {
+                kind: MatchKind::Novel {
+                    highlight: TokenKind::Atom(kind),
+                },
+                pos: *line,
+            })
+            .collect();
+    }
 
     let content_newlines = NewlinePositions::from(content);
     let opposite_content_newlines = NewlinePositions::from(opposite_content);
@@ -704,17 +735,18 @@ fn split_comment_words(
     let mut opposite_offset = 0;
 
     let mut res = vec![];
-    for diff_res in myers_diff::slice_by_hash(&content_parts, &other_parts) {
+    for diff_res in word_diffs {
         match diff_res {
             myers_diff::DiffResult::Left(word) => {
                 // This word is novel to this side.
                 if !is_all_whitespace(word) {
                     res.push(MatchedPos {
                         kind: MatchKind::NovelWord {
-                            highlight: TokenKind::Atom(AtomKind::Comment),
+                            highlight: TokenKind::Atom(kind),
                         },
                         pos: content_newlines.from_offsets_relative_to(
-                            pos,
+                            // TODO: don't assume a single line atom.
+                            pos[0],
                             offset,
                             offset + word.len(),
                         )[0],
@@ -724,17 +756,19 @@ fn split_comment_words(
             }
             myers_diff::DiffResult::Both(word, opposite_word) => {
                 // This word is present on both sides.
+                // TODO: don't assume this atom is on a single line.
                 let word_pos =
-                    content_newlines.from_offsets_relative_to(pos, offset, offset + word.len())[0];
+                    content_newlines.from_offsets_relative_to(pos[0], offset, offset + word.len())
+                        [0];
                 let opposite_word_pos = opposite_content_newlines.from_offsets_relative_to(
-                    opposite_pos,
+                    opposite_pos[0],
                     opposite_offset,
                     opposite_offset + opposite_word.len(),
                 );
 
                 res.push(MatchedPos {
                     kind: MatchKind::NovelLinePart {
-                        highlight: TokenKind::Atom(AtomKind::Comment),
+                        highlight: TokenKind::Atom(kind),
                         self_pos: word_pos,
                         opposite_pos: opposite_word_pos,
                     },
@@ -753,6 +787,35 @@ fn split_comment_words(
     res
 }
 
+/// Are there sufficient common words that we should only highlight
+/// individual changed words?
+fn has_common_words(word_diffs: &Vec<myers_diff::DiffResult<&&str>>) -> bool {
+    let mut novel_count = 0;
+    let mut unchanged_count = 0;
+
+    for word_diff in word_diffs {
+        match word_diff {
+            myers_diff::DiffResult::Both(word, _) => {
+                if **word != " " {
+                    unchanged_count += 1;
+                }
+            }
+            _ => {
+                novel_count += 1;
+            }
+        }
+    }
+
+    // We want more than two unchanged words, because the text content
+    // includes the comment or string delimiters.
+    //
+    // A sufficiently similar set of words is when more than 50% of
+    // the words are common between the two sides. We multiply by two
+    // because non-matching words gives us two novel words, whereas
+    // matched words only gives us one unchanged word.
+    unchanged_count > 2 && unchanged_count * 2 >= novel_count
+}
+
 impl MatchedPos {
     fn new(
         ck: ChangeKind,
@@ -761,7 +824,7 @@ impl MatchedPos {
         is_close: bool,
     ) -> Vec<Self> {
         match ck {
-            ReplacedComment(this, opposite) => {
+            ReplacedComment(this, opposite) | ReplacedString(this, opposite) => {
                 let this_content = match this {
                     List { .. } => unreachable!(),
                     Atom { content, .. } => content,
@@ -772,14 +835,13 @@ impl MatchedPos {
                         content, position, ..
                     } => (content, position),
                 };
+                let kind = if let ReplacedComment(_, _) = ck {
+                    AtomKind::Comment
+                } else {
+                    AtomKind::String
+                };
 
-                split_comment_words(
-                    this_content,
-                    // TODO: handle the whole pos here.
-                    pos[0],
-                    opposite_content,
-                    opposite_pos[0],
-                )
+                split_atom_words(this_content, pos, opposite_content, opposite_pos, kind)
             }
             Unchanged(opposite) => {
                 let opposite_pos = match opposite {
@@ -1117,34 +1179,142 @@ mod tests {
     }
 
     #[test]
-    fn test_split_comment_words_basic() {
-        let content = "abc";
-        let pos = SingleLineSpan {
+    fn test_split_atom_words() {
+        let content = "abc def ghi novel";
+        let pos = vec![SingleLineSpan {
             line: 0.into(),
             start_col: 0,
-            end_col: 3,
-        };
+            end_col: 17,
+        }];
 
-        let opposite_content = "def";
-        let opposite_pos = SingleLineSpan {
+        let opposite_content = "abc def ghi";
+        let opposite_pos = vec![SingleLineSpan {
             line: 0.into(),
             start_col: 0,
-            end_col: 3,
-        };
+            end_col: 11,
+        }];
 
-        let res = split_comment_words(content, pos, opposite_content, opposite_pos);
+        let res = split_atom_words(
+            content,
+            &pos,
+            opposite_content,
+            &opposite_pos,
+            AtomKind::Comment,
+        );
         assert_eq!(
             res,
-            vec![MatchedPos {
-                kind: MatchKind::NovelWord {
-                    highlight: TokenKind::Atom(AtomKind::Comment),
+            vec![
+                MatchedPos {
+                    kind: MatchKind::NovelLinePart {
+                        highlight: TokenKind::Atom(AtomKind::Comment),
+                        self_pos: SingleLineSpan {
+                            line: 0.into(),
+                            start_col: 0,
+                            end_col: 3
+                        },
+                        opposite_pos: vec![SingleLineSpan {
+                            line: 0.into(),
+                            start_col: 0,
+                            end_col: 3
+                        }]
+                    },
+                    pos: SingleLineSpan {
+                        line: 0.into(),
+                        start_col: 0,
+                        end_col: 3
+                    }
                 },
-                pos: SingleLineSpan {
-                    line: 0.into(),
-                    start_col: 0,
-                    end_col: 3
+                MatchedPos {
+                    kind: MatchKind::NovelLinePart {
+                        highlight: TokenKind::Atom(AtomKind::Comment),
+                        self_pos: SingleLineSpan {
+                            line: 0.into(),
+                            start_col: 3,
+                            end_col: 4
+                        },
+                        opposite_pos: vec![SingleLineSpan {
+                            line: 0.into(),
+                            start_col: 3,
+                            end_col: 4
+                        }]
+                    },
+                    pos: SingleLineSpan {
+                        line: 0.into(),
+                        start_col: 3,
+                        end_col: 4
+                    }
                 },
-            },]
+                MatchedPos {
+                    kind: MatchKind::NovelLinePart {
+                        highlight: TokenKind::Atom(AtomKind::Comment),
+                        self_pos: SingleLineSpan {
+                            line: 0.into(),
+                            start_col: 4,
+                            end_col: 7
+                        },
+                        opposite_pos: vec![SingleLineSpan {
+                            line: 0.into(),
+                            start_col: 4,
+                            end_col: 7
+                        }]
+                    },
+                    pos: SingleLineSpan {
+                        line: 0.into(),
+                        start_col: 4,
+                        end_col: 7
+                    }
+                },
+                MatchedPos {
+                    kind: MatchKind::NovelLinePart {
+                        highlight: TokenKind::Atom(AtomKind::Comment),
+                        self_pos: SingleLineSpan {
+                            line: 0.into(),
+                            start_col: 7,
+                            end_col: 8
+                        },
+                        opposite_pos: vec![SingleLineSpan {
+                            line: 0.into(),
+                            start_col: 7,
+                            end_col: 8
+                        }]
+                    },
+                    pos: SingleLineSpan {
+                        line: 0.into(),
+                        start_col: 7,
+                        end_col: 8
+                    }
+                },
+                MatchedPos {
+                    kind: MatchKind::NovelLinePart {
+                        highlight: TokenKind::Atom(AtomKind::Comment),
+                        self_pos: SingleLineSpan {
+                            line: 0.into(),
+                            start_col: 8,
+                            end_col: 11
+                        },
+                        opposite_pos: vec![SingleLineSpan {
+                            line: 0.into(),
+                            start_col: 8,
+                            end_col: 11
+                        }]
+                    },
+                    pos: SingleLineSpan {
+                        line: 0.into(),
+                        start_col: 8,
+                        end_col: 11
+                    }
+                },
+                MatchedPos {
+                    kind: MatchKind::NovelWord {
+                        highlight: TokenKind::Atom(AtomKind::Comment)
+                    },
+                    pos: SingleLineSpan {
+                        line: 0.into(),
+                        start_col: 12,
+                        end_col: 17
+                    }
+                }
+            ],
         );
     }
 
@@ -1156,12 +1326,19 @@ mod tests {
     }
 
     #[test]
-    fn test_split_words_punctuations() {
+    fn test_split_words_punctuation() {
         let s = "example..";
         let res = split_words(s);
         assert_eq!(res, vec!["example", ".", "."])
     }
 
+    #[test]
+    fn test_split_words_numbers() {
+        let s = "foo123bar";
+        let res = split_words(s);
+        assert_eq!(res, vec!["foo123bar"])
+    }
+
     #[test]
     fn test_split_words_treats_newline_separately() {
         let s = "example.\ncom";
@@ -1189,4 +1366,18 @@ mod tests {
         let res = split_words(s);
         assert_eq!(res, vec!["a", " ", "xöy", " ", "b"])
     }
+
+    #[test]
+    fn test_split_words_and_numbers() {
+        let s = "a123b";
+        let res = split_words_and_numbers(s);
+        assert_eq!(res, vec!["a", "123", "b"])
+    }
+
+    #[test]
+    fn test_split_words_and_numbers_spaces() {
+        let s = "foo bar";
+        let res = split_words_and_numbers(s);
+        assert_eq!(res, vec!["foo", " ", "bar"])
+    }
 }
