@@ -91,6 +91,7 @@ extern "C" {
     fn tree_sitter_json() -> ts::Language;
     fn tree_sitter_julia() -> ts::Language;
     fn tree_sitter_kotlin() -> ts::Language;
+    fn tree_sitter_latex() -> ts::Language;
     fn tree_sitter_lua() -> ts::Language;
     fn tree_sitter_make() -> ts::Language;
     fn tree_sitter_newick() -> ts::Language;
@@ -425,9 +426,13 @@ pub fn from_language(language: guess::Language) -> TreeSitterConfig {
             let language = unsafe { tree_sitter_haskell() };
             TreeSitterConfig {
                 language,
-                atom_nodes: vec!["qualified_variable", "qualified_module"]
-                    .into_iter()
-                    .collect(),
+                atom_nodes: vec![
+                    "qualified_variable",
+                    "qualified_module",
+                    "qualified_constructor",
+                ]
+                .into_iter()
+                .collect(),
                 delimiter_tokens: vec![("[", "]"), ("(", ")")],
                 highlight_query: ts::Query::new(
                     language,
@@ -608,6 +613,20 @@ pub fn from_language(language: guess::Language) -> TreeSitterConfig {
                 sub_languages: vec![],
             }
         }
+        LaTeX => {
+            let language = unsafe { tree_sitter_latex() };
+            TreeSitterConfig {
+                language,
+                atom_nodes: vec![].into_iter().collect(),
+                delimiter_tokens: vec![("{", "}"), ("[", "]")],
+                highlight_query: ts::Query::new(
+                    language,
+                    include_str!("../../vendored_parsers/highlights/latex.scm"),
+                )
+                .unwrap(),
+                sub_languages: vec![],
+            }
+        }
         Lua => {
             let language = unsafe { tree_sitter_lua() };
             TreeSitterConfig {
